import java.io.*;
import java.util.*;

import edu.uci.ics.jung.graph.*;

import net.deterlab.abac.Credential;
import net.deterlab.abac.CredentialGraph;
import net.deterlab.abac.Query;
import net.deterlab.abac.Role;
import net.deterlab.abac.Identity;

import org.bouncycastle.openssl.PEMReader;
import org.bouncycastle.jce.provider.X509CertificateObject;
import java.security.KeyPair;
import java.security.PublicKey;
// import org.bouncycastle.util.io.pem.PemObject;


/**
 * Simple test of the native Java implementation of ABAC. Loads credentials
 * from an rt0 file and runs a query against them.
 */
public class GraphTest {

    /**
     * Import a credential from a der file and add it to the credential graph.
     */
    protected static void importCred(File f, CredentialGraph g) 
	throws Exception {
	Credential c = new Credential(f);
	g.add_credential(c);
    }


    /**
     * Import a directory full of files, using the suffixes as determinants.
     * First import all the identities (pem), then the credentials (der) into
     * the credential graph then any alias files into the two maps.
     */
    protected static void importDir(File d, CredentialGraph g) {
	Vector<File> ids = new Vector<File>();
	Vector<File> creds = new Vector<File>();

	for (File f: d.listFiles()) {
	    if (f.getPath().endsWith(".pem")) ids.add(f);
	    else if (f.getPath().endsWith(".der") ) creds.add(f);
	    else System.out.println(f + " of unknown type");
	}
	for (File f: ids ){
	    try {
		Identity id = new Identity(f);

		Credential.addIdentity(id);
	    }
	    catch (Exception e) {
		System.err.println("Cannot add " + f + ": " + e);
	    }
	}

	for (File f: creds) {
	    try {
		importCred(f, g);
	    }
	    catch (Exception e) {
		System.err.println("Cannot add " + f + ": " + e);
	    }
	}
    }
    public static void main(String[] args) throws IOException {
        if (args.length < 3) {
            System.out.println("Usage: GraphTest <files> <role> <principal>");
            System.out.println("    runs the query role <-?- principal and prints the result");
            System.exit(1);
        }

        CredentialGraph graph = new CredentialGraph();
	TreeMap<String, String> nicknames = new TreeMap<String, String>();
	TreeMap<String, String> keys = new TreeMap<String, String>();

	for (int i= 0; i < args.length-2; i++) {
	    File f = new File(args[i]);

	    try {
		if (f.isDirectory()) 
		    importDir(f, graph);
		else if (f.getPath().endsWith(".pem")) {
		    Identity id = new Identity(f);
		    Credential.addIdentity(id);
		}
		else if (f.getPath().endsWith(".der"))
		    importCred(f, graph);
		else
		    System.out.println(f + " of unknown type");
	    }
	    catch (Exception e) {
		System.err.println("Failed to process " + f + ": " +e);
	    }
	}

        //
        // run the query
        //

	Role role = new Role(args[args.length-2], true);
	Role prin = new Role(args[args.length-1], true);
        Query q = graph.querier();
        Graph<Role, Credential> ret = q.run(role.toString(), prin.toString());
	String fn = "attr";
	int n = 0;
	String suf = ".der";
        for (Credential c : ret.getEdges()) {
            System.out.println(c.simpleString());
	    c.write(fn + n++ + suf);
	}

	fn = "id";
	n = 0;
	suf = ".pem";
	for (Identity i: Credential.identities()) {
	    System.out.println("ID: " + i);
	    i.write(fn + n++ + suf);
	}
    }
}
